/* University of Strasbourg - Master ILC-ISI-RISE
 * Compilation Lab - Compiler for the arith language
 * Written by Cedric Bastoul cedric.bastoul@unistra.fr
 */

%{
  #include <stdio.h>
  #include <stdlib.h>

  void yyerror(char*);
  int  yylex();
  void lex_free();
  extern FILE* yyin;
%}

%union {
  char* string;
  int value;
}

%token PRINT
%token <string> IDENTIFIER
%token <value>  NUMBER

%left '+'
%left '*'

%%

axiom:
    statement_list               { printf("[Y] Match :-) !\n"); return 0; }
  ;

statement_list:
    statement statement_list     { printf("[Y] stmt_lst -> stmt stmt_lst\n"); }
  | statement                    { printf("[Y] stmt_lst -> stmt\n"); }
  ;

statement:
    IDENTIFIER '=' expression ';'{ printf("[Y] stmt -> ID (%s) = expr;\n",$1);}
  | PRINT expression ';'         { printf("[Y] stmt -> PRINT expr;\n"); }
  ;

expression:
    expression '+' expression    { printf("[Y] expr -> expr + expr\n"); }
  | expression '*' expression    { printf("[Y] expr -> expr * expr\n"); }
  | '(' expression ')'           { printf("[Y] expr -> ( expr )\n"); }
  | IDENTIFIER                   { printf("[Y] expr -> ID (%s)\n", $1); }
  | NUMBER                       { printf("[Y] expr -> NUMBER (%d)\n", $1); }
  ;

%%

void yyerror (char *s) {
    printf("[Y] ERROR: %s\n", s);
}

int main(int argc, char* argv[]) {
  if (argc > 2) {
    fprintf(stderr, "usage: %s [file]\n", argv[0]);
    exit(1);
  }

  if (argc == 2) {
    yyin = fopen(argv[1], "r");
    if (yyin == NULL) {
      fprintf(stderr, "unable to open %s\n", argv[1]);
      exit(1);
    }
  }

  yyparse();

  // Be clean.
  lex_free();
  fclose(yyin);
  return 0;
}
